"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseExpressionBuilder = void 0;
exports.validatePagination = validatePagination;
exports.validateValue = validateValue;
const foundation_1 = require("@denis_bruns/foundation");
const MAX_VALUE_LENGTH = 400000;
class BaseExpressionBuilder {
    pkName;
    constructor(pkName = "id") {
        this.pkName = pkName;
    }
    extractPartitionKeyFilter(filters) {
        const remaining = [...filters];
        const pkIndex = remaining.findIndex((f) => f.field === this.pkName && f.operator === "=");
        if (pkIndex === -1) {
            return { remainingFilters: remaining };
        }
        const pkFilter = remaining[pkIndex];
        remaining.splice(pkIndex, 1);
        return { pkFilter, remainingFilters: remaining };
    }
}
exports.BaseExpressionBuilder = BaseExpressionBuilder;
function validatePagination(pagination = {}) {
    if (!pagination) {
        pagination = {};
    }
    const { page = 1, size = 10, limit, offset = 0 } = pagination;
    if (page !== undefined && !Number.isInteger(Number(page))) {
        throw new foundation_1.DynamoValidationError('Page must be an integer');
    }
    if (size !== undefined && !Number.isInteger(Number(size))) {
        throw new foundation_1.DynamoValidationError('Size must be an integer');
    }
    if (limit !== undefined && !Number.isInteger(Number(limit))) {
        throw new foundation_1.DynamoValidationError('Limit must be an integer');
    }
    if (offset !== undefined && !Number.isInteger(Number(offset))) {
        throw new foundation_1.DynamoValidationError('Offset must be an integer');
    }
}
function validateValue(value) {
    if (value === undefined || value === null) {
        throw new foundation_1.DynamoValidationError('Value cannot be null or undefined');
    }
    const stringified = JSON.stringify(value);
    const dangerousPatterns = [
        '$where', '$regex', '$ne', '$gt', '$lt', '$gte', '$lte',
        '$in', '$nin', '$or', '$and', '$not', '$exists', '$type',
        '$mod', '$text', '$elemMatch', '$size', '$all', '$expr',
        '__proto__', 'constructor', 'prototype'
    ];
    if (dangerousPatterns.some(pattern => stringified.toLowerCase().includes(pattern.toLowerCase()))) {
        throw new foundation_1.DynamoValidationError('Potential NoSQL injection detected');
    }
    if (typeof value === 'string' && value.length > MAX_VALUE_LENGTH) {
        throw new foundation_1.DynamoValidationError(`Value length exceeds maximum of ${MAX_VALUE_LENGTH}`);
    }
    if (Array.isArray(value)) {
        if (!value.length) {
            throw new foundation_1.DynamoValidationError('Empty arrays not supported');
        }
        value.forEach(validateValue);
    }
    else if (typeof value === 'object' && value !== null) {
        const propsToCheck = [...Object.keys(value), ...Object.getOwnPropertyNames(value)];
        if (propsToCheck.some(prop => dangerousPatterns.includes(prop))) {
            throw new foundation_1.DynamoValidationError('Potential NoSQL injection detected');
        }
        Object.values(value).forEach(validateValue);
    }
}
//# sourceMappingURL=BaseExpressionBuilder.js.map